Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 40773)
+++ channels/chan_sip.c	(working copy)
@@ -877,6 +877,7 @@
 		AST_STRING_FIELD(peername);	/*!< [peer] name, not set if [user] */
 		AST_STRING_FIELD(authname);	/*!< Who we use for authentication */
 		AST_STRING_FIELD(uri);		/*!< Original requested URI */
+		AST_STRING_FIELD(ruri);         /*!< SIP Request-URI */
 		AST_STRING_FIELD(okcontacturi);	/*!< URI from the 200 OK on INVITE */
 		AST_STRING_FIELD(peersecret);	/*!< Password */
 		AST_STRING_FIELD(peermd5secret);
@@ -10899,6 +10900,81 @@
 	.read = func_header_read,
 };
 
+/*! \brief Read SIP Request-URI parameter (dialplan function) */
+static int func_ruri_param_read(struct ast_channel *chan, char *function, char *data, char *buf, size_t len)
+{
+        struct sip_pvt *p;
+	char *pat_start = ";";
+	char *pat_end = "=([^;]+)";
+	char *pattern;
+	regmatch_t pmatch[2];
+	regex_t re;
+
+        ast_channel_lock(chan);
+        if (chan->tech != &sip_tech) {
+                ast_log(LOG_WARNING, "This function can only be used on SIP channels.\n");
+                ast_channel_unlock(chan);
+                return -1;
+        }
+
+        p = chan->tech_pvt;
+
+        /* If there is no private structure, this channel is no longer alive */
+        if (!p) {
+                ast_channel_unlock(chan);
+                return -1;
+        }
+
+	if (strlen(data) == 0) {
+		/* no data means we return full R-URI */
+		ast_copy_string(buf, p->ruri, strlen(p->ruri) + 1);
+		ast_channel_unlock(chan);
+		return 0;	
+	}
+
+	/* compile regex pattern: ;<data>=([^;]+) */
+	if (!(pattern = (char *)ast_calloc(strlen(pat_start) + strlen(pat_end) + strlen(data) + 1,
+                                           sizeof(char)))) {
+		ast_channel_unlock(chan);
+		return -1;
+	}
+	strcpy(pattern, pat_start);
+	strncat(pattern, data, strlen(data));
+	strcat(pattern, pat_end);
+	if (regcomp(&re, pattern, REG_EXTENDED) != 0) {
+		free(pattern);
+		regfree(&re);
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	/* apply pattern to p->ruri */
+	if (regexec(&re, p->ruri, (size_t) 2, pmatch, 0) == 0) {
+		/* pattern found, copy substring 1 into buf */
+		ast_copy_string(buf, p->ruri + pmatch[1].rm_so, pmatch[1].rm_eo - pmatch[1].rm_so + 1);
+	} else {
+		/* pattern not found */
+		regfree(&re);
+		free(pattern);
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	regfree(&re);
+	free(pattern);
+        ast_channel_unlock(chan);
+
+        return 0;
+}
+
+
+static struct ast_custom_function sip_ruri_param_function = {
+        .name = "SIPREQURI",
+        .synopsis = "Gets the specified SIP Request-URI parameter, SIPREQURI() returns full R-URI",
+        .syntax = "SIPREQURI(<name>)",
+        .read = func_ruri_param_read,
+};
+
 /*! \brief  Dial plan function to check if domain is local */
 static int func_check_sipdomain(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len)
 {
@@ -12595,6 +12671,9 @@
 		}
 	}
 
+	/* save Request-URI in p->ruri */
+	ast_string_field_set(p, ruri, e);
+	
 	/* Check if this is a loop */
 	if (ast_test_flag(&p->flags[0], SIP_OUTGOING) && p->owner && (p->owner->_state != AST_STATE_UP)) {
 		/* This is a call to ourself.  Send ourselves an error code and stop
@@ -16482,6 +16561,7 @@
 
 	/* Register dialplan functions */
 	ast_custom_function_register(&sip_header_function);
+	ast_custom_function_register(&sip_ruri_param_function);
 	ast_custom_function_register(&sippeer_function);
 	ast_custom_function_register(&sipchaninfo_function);
 	ast_custom_function_register(&checksipdomain_function);
@@ -16511,6 +16591,7 @@
 	ast_custom_function_unregister(&sipchaninfo_function);
 	ast_custom_function_unregister(&sippeer_function);
 	ast_custom_function_unregister(&sip_header_function);
+	ast_custom_function_unregister(&sip_ruri_param_function);
 	ast_custom_function_unregister(&checksipdomain_function);
 
 	ast_unregister_application(app_dtmfmode);
